@@ -3,11 +3,13 @@ package Net::OAI::Harvester;
 use strict;
 use warnings;
 
+use constant XMLNS_OAI => "http://www.openarchives.org/OAI/2.0/";
+
 use URI;
 use LWP::UserAgent;
 use XML::SAX qw( Namespaces Validation );
 use File::Temp qw( tempfile );
-use Carp qw( croak );
+use Carp qw( carp croak );
 
 use Net::OAI::Error;
 use Net::OAI::ResumptionToken;
@@ -16,13 +18,14 @@ use Net::OAI::ListMetadataFormats;
 use Net::OAI::ListIdentifiers;
 use Net::OAI::ListRecords;
 use Net::OAI::GetRecord;
-use Net::OAI::ListRecords;
 use Net::OAI::ListSets;
 use Net::OAI::Record::Header;
 use Net::OAI::Record::OAI_DC;
 
-our $VERSION = '1.15';
+our $VERSION = "1.20";
 our $DEBUG = 0;
+# compatibility mode for metadataHandler
+our $OLDmetadataHandler = 0;
 
 =head1 NAME
 
@@ -112,6 +115,7 @@ iterated over one at a time. The benefit of this is a lower memory footprint
 when (for example) a ListRecords verb is exercised on a repository that 
 returns 100,000 records.
 
+
 =item 2
 
 XML::SAX filters are used which will allow interested developers to write 
@@ -217,25 +221,24 @@ intersted in. For example:
     print "admin email(s): ", join( ", ", $identity->adminEmail() ), "\n";
     ...
 
-For more details see the Net::OAI::Identify documentation.
+For more details see the L<Net::OAI::Identify> documentation.
 
 =cut 
 
 sub identify {
     my $self = shift;
-    my $uri = $self->{ baseURL };
+    my $uri = $self->{ baseURL }->clone();
     $uri->query_form( 'verb' => 'Identify' );
 
     my $identity = Net::OAI::Identify->new( $self->_get( $uri ) );
-    return $identity if $identity->errorCode();
+    return $identity if $identity->{ error };
 
-    my $token = Net::OAI::ResumptionToken->new( Handler => $identity );
-    my $error = Net::OAI::Error->new( Handler => $token );
+    my $error = Net::OAI::Error->new( Handler => $identity );
     my $parser = _parser( $error ); 
     debug( "parsing Identify response " .  $identity->file() );
     eval { $parser->parse_uri( $identity->file() ) };
-    if ( $@ ) {_xmlError( $error ); } 
-    $identity->{ token } = $token->token() ? $token : undef;
+    if ( $@ ) { _xmlError( $error ); } 
+    $error->set_handler( undef );
     $identity->{ error } = $error;
     return( $identity );
 }
@@ -257,29 +260,27 @@ a particular resource identifier then you can pass in that identifier.
     print "record identifier 1234567 can be retrieved as ",
 	join( ',', $list->prefixes() ),"\n";
 
-See documentation for Net::OAI::ListMetadataFormats for more details.
+See documentation for L<Net::OAI::ListMetadataFormats> for more details.
 
 =cut
 
 sub listMetadataFormats {
     my ( $self, %opts ) = @_;
-    my $uri = $self->{ baseURL };
-    my %pairs = ( verb => 'ListMetadataFormats' );
-    if ( $opts{ identifier } ) { 
-	$pairs{ identifier } = $opts{ identifier }; 
-    }
-    $uri->query_form( %pairs );
+    my $uri = $self->{ baseURL }->clone();
+
+    $uri->query_form( verb => 'ListMetadataFormats', 
+        map { (defined $opts{$_}) ? ($_ => $opts{$_}) : () } qw( identifier )
+      );
 
     my $list = Net::OAI::ListMetadataFormats->new( $self->_get( $uri ) );
-    return $list if $list->errorCode();
+    return $list if $list->{ error };
 
-    my $token = Net::OAI::ResumptionToken->new( Handler => $list );
-    my $error = Net::OAI::Error->new( Handler => $token );
+    my $error = Net::OAI::Error->new( Handler => $list );
     my $parser = _parser( $error );
     debug( "parsing ListMetadataFormats response: ".$list->file() );
-    eval{ $parser->parse_uri( $list->file() ) };
+    eval { $parser->parse_uri( $list->file() ) };
     if ( $@ ) { _xmlError( $error ); } 
-    $list->{ token } = $token->token() ? $token : undef;
+    $error->set_handler( undef );
     $list->{ error } = $error;
     return( $list );
 }
@@ -290,66 +291,75 @@ getRecord() is used to retrieve a single record from a repository. You must pass
 in the C<identifier> and an optional C<metadataPrefix> parameters to identify 
 the record, and the flavor of metadata you would like. Net::OAI::Harvester 
 includes a parser for OAI DublinCore, so if you do not specifiy a 
-metadataPrefix 'oai_dc' will be assumed. If you would like to drop in you own 
-XML::Handler for another type of metadata use the C<metadataHandler> parameter.
+metadataPrefix 'oai_dc' will be assumed. If you would like to drop in your own 
+XML::Handler for another type of metadata use either the C<metadataHandler>
+or the C<recordHandler> parameter, either the name of the class as string
+or an already instantiated object of that class.
 
-    my $record = $harvester->getRecord( 
+    my $result = $harvester->getRecord( 
 	identifier	=> 'abc123',
     );
 
-    ## get the Net::OAI::Record::Header object
-    my $header = $record->header();
+    ## did something go wrong?
+    if ( my $oops = $result->errorCode() ) { ... };
+
+    ## get the result as Net::OAI::Record object
+    my $record = $result->record();     # undef if error
+
+    ## directly get the Net::OAI::Record::Header object
+    my $header = $result->header();     # undef if error
+    ## same as
+    my $header = $result->record()->header();     # undef if error
 
     ## get the metadata object 
-    my $metadata = $record->metadata();
+    my $metadata = $result->metadata(); # undef if error or harvested with recordHandler
 
     ## or if you would rather use your own XML::Handler 
     ## pass in the package name for the object you would like to create
-    my $record = $harvester->getRecord(
+    my $result = $harvester->getRecord(
 	identifier		=> 'abc123',
-	metadataHandler		=> 'MyHandler'	    
+	metadataHandler		=> 'MyHandler'
     );
-    my $metadata = $record->metadata();
+    my $metadata = $result->metadata();
+    
+    my $result = $harvester->getRecord(
+	identifier		=> 'abc123',
+	recordHandler		=> 'MyCompleteHandler'
+    );
+    my $complete_record = $result->recorddata(); # undef if error or harvested with metadataHandler
     
 =cut 
 
 sub getRecord {
     my ( $self, %opts ) = @_;
 
-    croak( "you must pass the identifier parameter to getRecord()" )
-	if ( ! exists( $opts{ 'identifier' } ) );
-    croak( "you must pass the metadataPrefix parameter to getRecord()" )
-	if ( ! exists( $opts{ 'metadataPrefix' } ) );
+    croak( "getRecord(): the 'identifier' parameter is required" )
+	unless defined $opts{ 'identifier' };
+    croak( "getRecord(): the 'metadataPrefix' parameter is required" )
+	unless exists $opts{ 'metadataPrefix' };
+    croak( "getRecord(): recordHandler and metadataHandler are mutually exclusive" )
+        if $opts{ recordHandler } and $opts{ metadataHandler };
 
-    my $metadataHandler;
-    if ( exists( $opts{ metadataHandler } ) ) {  
-	my $package = $opts{ metadataHandler };
-	_verifyMetadataHandler( $package );
-	$metadataHandler = $package->new();
-    } else {
-	$metadataHandler = Net::OAI::Record::OAI_DC->new();
-    }
+    my $uri = $self->{ baseURL }->clone();
 
-    my $uri = $self->{ baseURL };
-    $uri->query_form(
-	verb		=> 'GetRecord',
-	identifier	=> $opts{ 'identifier' },
-	metadataPrefix	=> $opts{ 'metadataPrefix' }
-    );
+    $uri->query_form( verb => 'GetRecord', 
+        map { (defined $opts{$_}) ? ($_ => $opts{$_}) : () } qw( identifier metadataPrefix )
+      );
 
-    my $record = Net::OAI::GetRecord->new( $self->_get( $uri ) );
-    return $record if $record->errorCode();
+    my $record = Net::OAI::GetRecord->new( $self->_get( $uri ), 
+	recordHandler => $opts{ recordHandler },
+	metadataHandler => $opts{ metadataHandler },
+        );
+    return $record if $record->{ error };
 
-    my $header = Net::OAI::Record::Header->new( Handler => $metadataHandler );
-    my $error = Net::OAI::Error->new( Handler => $header );
+    my $error = Net::OAI::Error->new( Handler => $record );
     my $parser = _parser( $error ); 
     debug( "parsing GetRecord response " . $record->file() );
-    $parser->parse_uri( $record->file() );
+    eval { $parser->parse_uri( $record->file() ) };
     if ( $@ ) { _xmlError( $error ); } 
 
+    $error->set_handler( undef );
     $record->{ error } = $error;
-    $record->{ metadata } = $metadataHandler;
-    $record->{ header } = $header;
     return( $record );
 
 }
@@ -375,7 +385,20 @@ OAI-PMH spec.
     }
 
 If you would like to use your own metadata handler then you can specify 
-the package name of the handler. 
+the package name of the handler as the C<metadataHandler> (will be exposed
+to events below the C<metadata> element) or C<recordHandler> (will be
+exposed to the C<record> element and all its children) parameter, passing
+either
+
+=over 4
+
+=item the name of the class as string, in that case a new instance
+will be created for any OAI record encountered or
+
+=item an already instantiated object of that class which will 
+be reused for all records. 
+
+=back
 
     my $records = $harvester->listRecords(
 	metadataPrefix	=> 'mods',
@@ -387,20 +410,26 @@ the package name of the handler.
 	# $metadata will be a MODS::Handler object
     }
 
-If you want to automatically handle resumption tokens you can with 
-the listAllRecords() method.
+If you want to automatically handle resumption tokens you can achieve
+this with the listAllRecords() method. In this case the C<next()> 
+method transparently causes the next response to be fetched from
+the repository if the current response ran out of records and 
+contained a resumptionToken.
 
 If you prefer you can handle resumption tokens yourself with a 
 loop, and the resumptionToken() method. You might want to do this
 if you are working with a repository that wants you to wait between
-requests.
+requests or if connectivity problems become an issue during particulary
+long harvesting runs and you want to implement a retransmission
+strategy for failing requests.
 
     my $records = $harvester->listRecords( metadataPrefix => 'oai_dc' );
+    my $responseDate = $records->responseDate();
     my $finished = 0;
 
     while ( ! $finished ) {
 
-	while ( my $record = $records->next() ) {
+	while ( my $record = $records->next() ) { # a Net::OAI::Record object
 	    my $metadata = $record->metadata();
 	    # do interesting stuff here 
 	}
@@ -416,29 +445,37 @@ requests.
 
     }
 
+Please note: Since C<listRecords()> stashes away the individual
+records it encounters with C<Storable>, special care has to 
+be taken if the handlers you provided make use of XS modules
+since these objects cannot be reliably handled. Therefore you will
+have to provide the special serializing and deserializing methods
+C<STORABLE_freeze()> and C<STORABLE_thaw()> for the objects
+used by your filter(s).
+
+
 =cut
 
 sub listRecords {
     my ( $self, %opts ) = @_;
 
-    croak( "you must pass the metadataPrefix parameter to listRecords()" )
-	if ( ! exists( $opts{ 'metadataPrefix' } )
-	    and ! exists( $opts{ 'resumptionToken' } ) );
-    my %pairs = ( 
-	verb		  => 'ListRecords', 
-    );
+    croak( "listRecords(): the 'metadataPrefix' parameter is required" )
+	unless ( exists $opts{ 'metadataPrefix' } 
+              or defined $opts{ 'resumptionToken' } );
+    croak( "listRecords(): recordHandler and metadataHandler are mutually exclusive" )
+        if $opts{ recordHandler } and $opts{ metadataHandler };
 
-    foreach ( qw( metadataPrefix from until set resumptionToken ) ) {
-	if ( exists( $opts{ $_ } ) ) {
-	    $pairs{ $_ } = $opts{ $_ };
-	}
-    }
-    my $uri = $self->{ baseURL };
-    $uri->query_form( %pairs );
+    my $uri = $self->{ baseURL }->clone();
+
+    $uri->query_form( verb => 'ListRecords', 
+        map { (defined $opts{$_}) ? ($_ => $opts{$_}) : () } qw( metadataPrefix from until set resumptionToken )
+      );
 
     my $list = Net::OAI::ListRecords->new( $self->_get( $uri ), 
-	metadataHandler => $opts{ metadataHandler } );
-    return $list if $list->errorCode();
+	metadataHandler => $opts{ metadataHandler },
+	recordHandler => $opts{ recordHandler },
+        );
+    return $list if $list->{ error };
 
     my $token = Net::OAI::ResumptionToken->new( Handler => $list );
     my $error = Net::OAI::Error->new( Handler => $token );
@@ -447,15 +484,27 @@ sub listRecords {
     eval { $parser->parse_uri( $list->file() ) };
     if ( $@ ) { _xmlError( $error ); } 
 
-    $list->{ error } = $error;
+    $token->set_handler( undef );
     $list->{ token } = $token->token() ? $token : undef;
+
+    $error->set_handler( undef );
+    $list->{ error } = $error;
+
     return( $list );
 }
 
 =head2 listAllRecords() 
 
-Does exactly what listRecords() does except it will automatically
-submit resumption tokens as needed.
+Does exactly what listRecords() does except the C<next()> method 
+will automatically submit resumption tokens as needed.
+
+    my $records = $harvester->listAllRecords( metadataPrefix => 'oai_dc' );
+
+    while ( my $record = $records->next() ) { # a Net::OAI::Record object until undef
+	my $metadata = $record->metadata();
+	# do interesting stuff here 
+    }
+
 
 =cut
 
@@ -472,18 +521,18 @@ sub listAllRecords {
 listIdentifiers() takes the same parameters that listRecords() takes, but it 
 returns only the record headers, allowing you to quickly retrieve all the 
 record identifiers for a particular repository. The object returned is a 
-Net::OAI::ListIdentifiers object.
+L<Net::OAI::ListIdentifiers> object.
 
     my $headers = $harvester->listIdentifiers( 
 	metadataPrefix	=> 'oai_dc'
     );
 
     ## iterate through the results with next()
-    while ( my $header = $identifiers->next() ) { 
+    while ( my $header = $identifiers->next() ) {  # a Net::OAI::Record::Header object
 	print "identifier: ", $header->identifier(), "\n";
     }
 
-If you want to automtically handle resumption tokens use listAllIdentifiers().
+If you want to automatically handle resumption tokens use listAllIdentifiers().
 If you are working with a repository that encourages pauses between requests
 you can handle the tokens yourself using the technique described above
 in listRecords().
@@ -492,22 +541,17 @@ in listRecords().
 
 sub listIdentifiers {
     my ( $self, %opts ) = @_;
-    croak( "listIdentifiers(): metadataPrefix is a required parameter" ) 
-	if ( ! exists( $opts{ metadataPrefix } )
-	    and ! exists( $opts{ resumptionToken } ) );
-    my $uri = $self->{ baseURL };
-    my %pairs = (
-	verb		=> 'ListIdentifiers', 
-    );
-    foreach ( qw( metadataPrefix from until set resumptionToken ) ) {
-	if ( exists( $opts{ $_ } ) ) {
-	    $pairs{ $_ } = $opts{ $_ };
-	}
-    }
-    $uri->query_form( %pairs );
+    croak( "listIdentifiers(): the 'metadataPrefix' parameter is required" )
+	unless ( exists $opts{ 'metadataPrefix' } 
+              or defined $opts{ 'resumptionToken' } );
+    my $uri = $self->{ baseURL }->clone();
+
+    $uri->query_form( verb => 'ListIdentifiers', 
+        map { (defined $opts{$_}) ? ($_ => $opts{$_}) : () } qw( metadataPrefix from until set resumptionToken )
+      );
 
     my $list = Net::OAI::ListIdentifiers->new( $self->_get( $uri ) );
-    return( $list ) if $list->errorCode();
+    return( $list ) if $list->{ error };
 
     my $token = Net::OAI::ResumptionToken->new( Handler => $list );
     my $error = Net::OAI::Error->new( Handler => $token );
@@ -515,14 +559,19 @@ sub listIdentifiers {
     debug( "parsing ListIdentifiers response " . $list->file() );
     eval { $parser->parse_uri( $list->file() ) };
     if ( $@ ) { _xmlError( $error ); } 
+
+    $token->set_handler( undef );
     $list->{ token } = $token->token() ? $token : undef; 
+
+    $error->set_handler( undef );
     $list->{ error } = $error;
+
     return( $list );
 }
 
 =head2 listAllIdentifiers()
 
-Does exactly what listIdentifiers() does except it will automatically 
+Does exactly what listIdentifiers() does except C<next()> will automatically 
 submit resumption tokens as needed.
 
 =cut
@@ -535,6 +584,7 @@ sub listAllIdentifiers {
     return( $list );
 }
 
+
 =head2 listSets()
 
 listSets() takes an optional C<resumptionToken> parameter, and returns a 
@@ -551,30 +601,33 @@ spec and the Net::OAI::ListSets docs.
 
 sub listSets {
     my ( $self, %opts ) = @_;
-    my %pairs = ( verb => 'ListSets' );
-    if ( exists( $opts{ resumptionToken } ) ) {
-	$pairs{ resumptionToken } = $opts{ resumptionToken };
-    }
-    my $uri = $self->{ baseURL };
-    $uri->query_form( %pairs );
+
+    my $uri = $self->{ baseURL }->clone();
+
+    $uri->query_form( verb => 'ListSets', 
+        map { (defined $opts{$_}) ? ($_ => $opts{$_}) : () } qw( resumptionToken )
+      );
 
     my $list = Net::OAI::ListSets->new( $self->_get( $uri ) );
-    return( $list ) if $list->errorCode();
+    return( $list ) if $list->{ error };
 
     my $token = Net::OAI::ResumptionToken->new( Handler => $list );
     my $error = Net::OAI::Error->new( Handler => $token );
     my $parser = _parser( $error );
     debug( "parsing ListSets response " . $list->file() );
-    eval{ $parser->parse_uri( $list->file() ) };
+    eval { $parser->parse_uri( $list->file() ) };
     if ( $@ ) { _xmlError( $error ); } 
-    $list->{ error } = $error;
+
+    $token->set_handler( undef );
     $list->{ token } = $token->token() ? $token : undef;
+    $error->set_handler( undef );
+    $list->{ error } = $error;
     return( $list );
 }
 
 =head2 baseURL()
 
-Gets or sets the base URL for the repository being harvested.
+Gets or sets the base URL for the repository being harvested (as L<URI/>).
 
     $harvester->baseURL( 'http://memory.loc.gov/cgi-bin/oai2_0' );
 
@@ -587,12 +640,13 @@ Or if you want to know what the current baseURL is
 sub baseURL {
     my ( $self, $url ) = @_;
     if ( $url ) { $self->{ baseURL } = URI->new( $url ); } 
-# The HTTP UserAgent modifies its URI object upon execution,
-# therefore we have to reconstruct: trim the query part ...
-    my $c = $self->{ baseURL }->canonical();
-    if ( $c && ($c =~ /^([^\?]*)\?/) ) {  # $c might be undefined
-        return $1};
-    return $c;
+## The HTTP UserAgent modifies its URI object upon execution,
+## therefore we'll always provide a clone <s>have to reconstruct: trim the query part ...</s>
+#    my $c = $self->{ baseURL };           # ->canonical();
+#    if ( $c && ($c =~ /^([^\?]*)\?/) ) {  # $c might be undefined
+#        return $1};
+#    return $c;
+    return $self->{ baseURL };
 }
 
 =head2 userAgent()
@@ -606,8 +660,7 @@ agent string, timeout, or some other feature.
 sub userAgent {
     my ( $self, $ua ) = @_;
     if ( $ua ) { 
-	croak( "userAgent() needs a valid LWP::UserAgent" ) 
-	    if ref( $ua ) ne 'LWP::UserAgent';
+	$ua->isa('LWP::UserAgent') or croak( "userAgent() needs a valid LWP::UserAgent" );
 	$self->{ userAgent } = $ua;
     }
     return( $self->{ userAgent } );
@@ -620,21 +673,18 @@ sub _get {
     my $ua = $self->{ userAgent };
 
     my ($fh, $file);
-
     if ( $self->{ dumpDir } ) {
         my $filePrefix = $self->{lastDump}++;
         $file = sprintf("%s/%08d.xml", $self->{dumpDir}, $filePrefix);
         $fh = IO::File->new($file, 'w');
-    }
-
-    else {
+    } else {
         ( $fh, $file ) = tempfile(UNLINK => 1);
     }
 
     debug( "fetching ".$uri->as_string() );
     debug( "writing to file: $file" );
     my $request = HTTP::Request->new( GET => $uri->as_string() );
-    my $response = $ua->request( $request, sub { print $fh shift; }, 4096 );
+    my $response = $ua->request( $request, sub { print $fh shift; }, 8192 );
     close( $fh );
 
     if ( $response->is_error() ) { 
@@ -646,12 +696,31 @@ sub _get {
             errorString     => 'HTTP Level Error: ' . $response->message(),
             errorCode       => $response->code(),
             HTTPError       => $response,
+            HTTPRetryAfter  => $response->header("Retry-After") || "",
         );
 	return( 
 #	    file	    => $file, 
             error           => $error
 	);
     }
+    if ( my $ct = $response->header("Content-Type") ) {
+        debug( "Content-type $ct in HTTP response" );
+        unless ( $ct =~ /^text\/xml(;|$)/ ) {
+            return (error => Net::OAI::Error->new(errorCode => 'xmlContentError',
+                                                errorString => "Content-Type: text/xml is mandatory (got: $ct)!"),
+                                                  HTTPError => $response,
+                                            HTTPRetryAfter  => $response->header("Retry-After") || "",
+                   )
+          };
+        if ( $ct =~ /; charset=(\S+)/ ) {
+            my $cs = $1;
+            return (error => Net::OAI::Error->new(errorCode => 'xmlContentError',
+                                                errorString => "charset=UTF-8 is mandatory (got: $cs)!"),
+                                                  HTTPError => $response,
+                                            HTTPRetryAfter  => $response->header("Retry-After") || "",
+                   ) unless $cs =~ /^utf-8/i;
+          };
+    }
 
     return( 
 	    file	    => $file,
@@ -665,19 +734,19 @@ sub _parser {
     my $parser;
     $factory->require_feature(Namespaces);
     eval { $parser = $factory->parser( Handler => $handler ) };
-    warn ref($factory)." threw an exception:\n\t$@" if $@;
+    carp ref($factory)." threw an exception:\n\t$@" if $@;
 
     if ( $parser && ref($parser) ) {
         debug( "using SAX parser " . ref($parser) . " " . $parser->VERSION );
         return $parser;
       };
 
-    warn "!!! Please check your setup of XML::SAX, especially ParserDetails.ini !!!\n";
+    carp "!!! Please check your setup of XML::SAX, especially ParserDetails.ini !!!\n";
     local($XML::SAX::ParserPackage) = "XML::SAX::PurePerl";
     eval { $parser = $factory->parser( Handler => $handler ) };
-    warn ref($factory)." threw an exception again:\n\t$@" if $@;
+    carp ref($factory)." threw an exception again:\n\t$@" if $@;
     if ( $parser && ref($parser) ) {
-        warn "Successfuly forced assignment of a parser: " . ref($parser) . " " . $parser->VERSION ."\n";
+        carp "Successfuly forced assignment of a parser: " . ref($parser) . " " . $parser->VERSION ."\n";
         return $parser;
       };
 
@@ -686,34 +755,38 @@ sub _parser {
 
 sub _xmlError {
     my $e = shift;
-    warn "caught xml parsing error: $@";
+    carp "caught xml parsing error: $@";
     $e->errorString( "XML parsing error: $@" );
     $e->errorCode( 'xmlParseError' );
 }
 
 
-sub _verifyMetadataHandler {
-    my $package = shift;
-    eval( "use $package" );
-    _fatal( "unable to locate metadataHandler $package in: " . 
-	join( "\n\t", @INC ) ) if $@; 
-    _fatal( "metadataHandler $package must inherit from XML::SAX::Base\n" )
-	if ( ! grep { 'XML::SAX::Base' } eval( '@' . $package . '::ISA' ) );
+sub _verifyHandler {
+    my $package_or_instance = shift;
+    if ( ref($package_or_instance) ) {
+        $package_or_instance->isa('XML::SAX::Base')
+            or _fatal( "Handler $package_or_instance must inherit from XML::SAX::Base\n" )
+      }
+    else {
+        eval( "use $package_or_instance" );
+        _fatal( "unable to locate Handler $package_or_instance in: " . 
+	    join( "\n\t", @INC ) ) if $@; 
+        _fatal( "Handler $package_or_instance must inherit from XML::SAX::Base\n" )
+            if ( ! grep { 'XML::SAX::Base' } eval( '@' . $package_or_instance . '::ISA' ) );
+      }
     return( 1 );
 }
 
 
 sub debug {
+    return unless $Net::OAI::Harvester::DEBUG;
     my $msg = shift; 
-    if ( $Net::OAI::Harvester::DEBUG ) { 
-	print STDERR "oai-harvester: " . localtime() . ": $msg\n";
-    }
+    carp "oai-harvester: " . localtime() . ": $msg\n";
 }
 
 sub _fatal {
     my $msg = shift;
-    print STDERR "fatal: $msg\n\n";
-    exit( 1 );
+    croak "fatal: $msg";
 }
 
 =head1 DIAGNOSTICS
@@ -723,6 +796,9 @@ then set Net::OAI::Harvester::DEBUG to a true value.
 
     $Net::OAI::Harvester::DEBUG = 1;
 
+
+
+
 =head1 PERFORMANCE 
 
 XML::SAX is used for parsing, but it presents a generalized interface to many 
@@ -735,6 +811,14 @@ If you have a particular parser you want to use you can set the
 $XML::SAX::ParserPackage variable appropriately. See XML::SAX::ParserFactory
 documentation for details.
 
+
+
+=head1 ENVIRONMENT
+
+The modules use LWP for HTTP operations, thus C<PERL_LWP_ENV_PROXY> controls
+wether the "_proxy" environment settings shall be honored.
+
+
 =head1 TODO
 
 =over 4 
@@ -770,6 +854,11 @@ Create common handlers for other metadata formats (MARC, qualified DC, etc).
 
 =item *
 
+Or at least provide a generic record handler as fallback, since using
+L<Net::OAI::Record::OAI_DC> makes absolutely no sense except for ... oai_dc records.
+
+=item *
+
 Selectively load Net::OAI::* classes as needed, rather than getting all of them 
 at once at the beginning of Net::OAI::Harvester.
 
@@ -825,7 +914,15 @@ L<Net::OAI::Record::Header>
 
 =item *
 
-L<Net::OAI::Record::Metadata>
+L<Net::OAI::Record::OAI_DC>
+
+=item *
+
+L<Net::OAI::Record::DocumentHelper>
+
+=item *
+
+L<Net::OAI::Record::NamespaceFilter>
 
 =item *
 
@@ -837,20 +934,20 @@ L<Storable>
 
 =back
 
+
 =head1 AUTHORS
 
-=over 4
-
-=item * 
-
 Ed Summers <ehs@pobox.com>
 
-=item *
-
 Martin Emmerich <Martin.Emmerich@oew.de>
 
-=back
+Thomas Berger <ThB@gymel.com>
+
+=head1 LICENSE
+
+This is free software, you may use it and distribute it under the same terms as Perl itself.
 
 =cut
 
 1;
+
