@@ -1,8 +1,8 @@
 package Net::OAI::Identify;
 
 use strict;
-use base qw( XML::SAX::Base );
-use base qw( Net::OAI::Base );
+use warnings;
+use base qw( XML::SAX::Base Net::OAI::Base );
 
 =head1 NAME
 
@@ -31,11 +31,11 @@ sub new {
     $self->{ adminEmails } = [];
     $self->{ compression } = '';
     $self->{ compressions } = [];
-    $self->{ insideDescription } = 0;
+    $self->{ _insideDescription } = 0;
     return( $self );
 }
 
-=head1 repositoryName() 
+=head2 repositoryName() 
 
 Returns the name of the repostiory.
 
@@ -46,7 +46,7 @@ sub repositoryName {
     return( $self->{ repositoryName } );
 }
 
-=head1 baseURL()
+=head2 baseURL()
 
 Returns the base URL used by the repository.
 
@@ -57,7 +57,7 @@ sub baseURL {
     return( $self->{ baseURL } );
 }
 
-=head1 protocolVersion()
+=head2 protocolVersion()
 
 Returns the version of the OAI-PMH used by the repository.
 
@@ -68,7 +68,7 @@ sub protocolVersion {
     return( $self->{ protocolVersion } );
 }
 
-=head1 earliestDatestamp()
+=head2 earliestDatestamp()
 
 Returns the earlies datestamp for records available in the repository.
 
@@ -79,7 +79,7 @@ sub earliestDatestamp {
     return( $self->{ earliestDatestamp } );
 }
 
-=head1 deletedRecord()
+=head2 deletedRecord()
 
 Indicates the way the repository works with deleted records. Should
 return I<no>, I<transient> or I<persistent>.
@@ -91,7 +91,7 @@ sub deletedRecord {
     return( $self->{ deletedRecord } );
 }
 
-=head1 granularity()
+=head2 granularity()
 
 Returns the granularity used by the repository.
 
@@ -102,7 +102,7 @@ sub granularity {
     return( $self->{ granularity } );
 }
 
-=head1 adminEmail()
+=head2 adminEmail()
 
 Returns the administrative email address for the repository. Since the 
 adminEmail elelemnt is allowed to repeat you will get all the emails (if more 
@@ -119,7 +119,7 @@ sub adminEmail {
     return( $self->{ adminEmails }[ 0 ] );
 }
 
-=head1 compression() {
+=head2 compression()
 
 Returns the types of compression that the archive supports. Since the 
 compression element may repeat you may get all the values by using 
@@ -136,36 +136,50 @@ sub compression {
     return( $self->{ compressions }[ 0 ] );
 }
 
+=head1 TODO
+
+Support for the various C<description> containers.
+
+=cut
+
 ## SAX Handlers
 
 sub start_element {
     my ( $self, $element ) = @_;
-    push( @{ $self->{ tagStack } }, $element->{ Name } );
-    $self->{ insideDescription } = 1 if $element->{ Name } eq 'description';
+    return $self->SUPER::start_element($element) unless $element->{NamespaceURI} eq Net::OAI::Harvester::XMLNS_OAI;  # should be error?
+
+    push( @{ $self->{ tagStack } }, $element->{ LocalName } );
+    $self->{ _insideDescription } = 1 if $element->{ LocalName } eq 'description';
 }
 
 sub end_element {
     my ( $self, $element ) = @_;
+    return $self->SUPER::end_element($element) unless $element->{NamespaceURI} eq Net::OAI::Harvester::XMLNS_OAI;  # should be error?
 
     ## store and reset elements that can have multiple values
-    if ( $element->{ Name } eq 'adminEmail' ) {
+    if ( $element->{ LocalName } eq 'adminEmail' ) {
         Net::OAI::Harvester::debug( "got adminEmail in Identify" );
 	push( @{ $self->{ adminEmails } }, $self->{ adminEmail } );
 	$self->{ adminEmail } = '';
     }
-    elsif ( $element->{ Name } eq 'compression' ) { 
+    elsif ( $element->{ LocalName } eq 'compression' ) { 
         Net::OAI::Harvester::debug( "got compression in Identify" );
 	push( @{ $self->{ compressions } }, $self->{ compression } );
 	$self->{ compression } = '';
     }
     pop( @{ $self->{ tagStack } } );
-    $self->{ insideDescription } = 0 if $element->{ Name } eq 'description';
+    $self->{ _insideDescription } = 0 if $element->{ LocalName } eq 'description';
 }
 
 sub characters {
     my ( $self, $characters ) = @_;
-    $self->{ $self->{ tagStack }[-1] } .= $characters->{ Data } 
-        unless $self->{ insideDescription };
+
+    if ( $self->{ _insideDescription } ) {
+        return $self->SUPER::characters( $characters );
+    } else {
+        $self->{ $self->{ tagStack }[-1] } .= $characters->{ Data } 
+    }
 }
 
 1;
+
