@@ -1,8 +1,9 @@
 package Net::OAI::ListIdentifiers;
 
 use strict;
-use base qw( XML::SAX::Base );
-use base qw( Net::OAI::Base );
+use warnings;
+use base qw( XML::SAX::Base Net::OAI::Base );
+use Carp qw ( croak );
 use Net::OAI::Record::Header;
 use File::Temp qw( tempfile );
 use IO::File;
@@ -18,6 +19,9 @@ Net::OAI::ListIdentifiers - Results of the ListIdentifiers OAI-PMH verb.
 
 =head1 METHODS
 
+This class is based on C<Net::OAI::Base> and inherits the general methods
+expect for the following:
+
 =head2 new()
 
 =cut
@@ -38,19 +42,23 @@ sub new {
 
 =head2 next()
 
+Returns the L<Net::OAI::Record::Header> object for the next OAI record in the
+response, C<undef> if none remain. resumptionToken handling is performed
+automagically if the original request was listAllIdentifiers().
+
 =cut
 
 sub next { 
     my $self = shift;
 
     if ( ! $self->{ headerFileHandle } ) {
-	$self->{ headerFileHandle } = 
-	    IO::File->new( $self->{ headerFilename } )
-	    || die "unable to open temp file: ".$self->{ headerFilename };
+	$self->{ headerFileHandle } = IO::File->new( $self->{ headerFilename } )
+	    or croak "unable to open temp file: ".$self->{ headerFilename };
     }
 
     if ( $self->{ headerFileHandle }->eof() ) {
-	$self->{ headerFileHandle }->close();
+	$self->{ headerFileHandle }->close() or croak "Could not close() ".$self->{ headerFilename }
+                                                      .". File system full?";
 	return( $self->handleResumptionToken( 'listIdentifiers' ) );
     }
 
@@ -62,9 +70,12 @@ sub next {
 
 sub start_element {
     my ( $self, $element ) = @_;
-    if ( $element->{ Name } eq 'header' ) {
+    return $self->SUPER::start_element( $element ) unless $element->{NamespaceURI} eq Net::OAI::Harvester::XMLNS_OAI;
+
+    if ( $element->{ LocalName } eq 'header' ) {
 	$self->{ OLD_Handler } = $self->get_handler();
 	$self->set_handler( Net::OAI::Record::Header->new() );
+    } elsif ( $element->{ LocalName } eq 'ListIdentifiers' ) {
     }
     $self->SUPER::start_element( $element );
 }
@@ -72,12 +83,14 @@ sub start_element {
 sub end_element {
     my ( $self, $element ) = @_;
     $self->SUPER::end_element( $element );
-    if ( $element->{ Name } eq 'header' ) {
+    return unless $element->{NamespaceURI} eq Net::OAI::Harvester::XMLNS_OAI;
+
+    if ( $element->{ LocalName } eq 'header' ) {
 	my $header = $self->get_handler();
         Net::OAI::Harvester::debug( "committing header to object store" );
 	store_fd( $header, $self->{ headerFileHandle } );
 	$self->set_handler( $self->{ OLD_Handler } );
-    } elsif ( $element->{ Name } eq 'ListIdentifiers' ) {
+    } elsif ( $element->{ LocalName } eq 'ListIdentifiers' ) {
         Net::OAI::Harvester::debug( "finished reading identifiers" );
 	$self->{ headerFileHandle }->close();
 	$self->{ headerFileHandle } = undef;
