@@ -1,8 +1,8 @@
 package Net::OAI::ListSets;
 
 use strict;
-use base qw( XML::SAX::Base );
-use base qw( Net::OAI::Base );
+use warnings;
+use base qw( XML::SAX::Base Net::OAI::Base );
 
 =head1 NAME
 
@@ -21,7 +21,6 @@ Net::OAI::ListSets - The results of the ListSets OAI-PMH verb.
 sub new {
     my ( $class, %opts ) = @_;
     my $self = bless \%opts, ref( $class ) || $class;
-    $self->{ insideSet } = 0;
     $self->{ specs } = {};
     return( $self );
 }
@@ -57,18 +56,19 @@ sub setName {
 sub start_element {
     my ( $self, $element ) = @_;
     $self->SUPER::start_element( $element );
-    push( @{ $self->{ tagStack } }, $element->{ Name } );
+    return unless $element->{ NamespaceURI } eq Net::OAI::Harvester::XMLNS_OAI;
+    push( @{ $self->{ tagStack } }, $element->{ LocalName } );
 }
 
 sub end_element {
     my ( $self, $element ) = @_;
-    if ( $element->{ Name } eq 'set' ) { 
-	$self->{ specs }{ $self->{ setSpec } } = $self->{ setName };
-	$self->{ setSpec } = undef;
-	$self->{ setName } = undef;
-    }
     $self->SUPER::end_element( $element );
+    return unless $element->{ NamespaceURI } eq Net::OAI::Harvester::XMLNS_OAI;
     pop( @{ $self->{ tagStack } } );
+    if ( $element->{ LocalName } eq 'set' ) { 
+	$self->{ specs }{ $self->{ setSpec } } = $self->{ setName };
+	$self->{ setSpec } = $self->{ setName } = undef;
+    }
 }
 
 sub characters {
